@@ -1,5 +1,6 @@
 # SPDX-License-Identifer: Apache-2.0
 # Copyright 2021 The Meson development team
+from __future__ import annotations
 
 from .common import cmake_is_debug
 from .. import mlog
@@ -49,8 +50,8 @@ def resolve_cmake_trace_targets(target_name: str,
                 res.libraries += [curr]
             elif Path(curr).is_absolute() and Path(curr).exists():
                 res.libraries += [curr]
-            elif env.machines.build.is_windows() and reg_is_maybe_bare_lib.match(curr) and clib_compiler:
-                # On Windows, CMake library dependencies can be passed as bare library names,
+            elif reg_is_maybe_bare_lib.match(curr) and clib_compiler:
+                # CMake library dependencies can be passed as bare library names,
                 # CMake brute-forces a combination of prefix/suffix combinations to find the
                 # right library. Assume any bare argument passed which is not also a CMake
                 # target must be a system library we should try to link against.
